﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Design;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.SqlServer.Dts.Runtime.Design;

namespace ToddMcDermid.SSIS
{
    /// <summary>
    /// Class to provide an interface to the main UI
    /// for the component
    /// </summary>
    public class ReplacingDataConversionUI : IDtsComponentUI
    {
        private IDTSComponentMetaData100 _componentMetadata = null;
        private IServiceProvider _serviceProvider = null;
        private IDtsConnectionService _connService = null;

        /// <summary>
        /// Initializes the user interface of the component
        /// </summary>
        /// <param name="dtsComponentMetadata">The underlying component metadata</param>
        /// <param name="serviceProvider">The service provider provided by the SSIS designer</param>
        /// <remarks>
        /// This method is called by SSIS when a component is initially 
        /// added to the SSIS Designer surface, and prior to the New method. 
        /// It is also called when a component is edited, but prior to the Edit method.
        /// </remarks>
        public void Initialize(IDTSComponentMetaData100 dtsComponentMetadata, IServiceProvider serviceProvider)
        {
            this._componentMetadata = dtsComponentMetadata;
            this._serviceProvider = serviceProvider;
            this._connService = (IDtsConnectionService)_serviceProvider.GetService(typeof(IDtsConnectionService));
        }

        #region IDtsComponentUI Members

        /// <summary>
        /// Handles when the component is deleted from the SSIS design surface,
        /// not currently used for anything
        /// </summary>
        /// <param name="parentWindow">The handle of the parent window hosting the component</param>
        public void Delete(IWin32Window parentWindow)
        {

        }

        /// <summary>
        /// Loads the user interface for when the component is edited
        /// </summary>
        /// <param name="parentWindow">The handle of the parent window hosting the component</param>
        /// <param name="variables">The collection of variables accessible to the data flow task containing the component</param>
        /// <param name="connections">The collection of connections accessible to the data flow task containing the component</param>
        /// <returns>True if the component has been modified</returns>
        public bool Edit(IWin32Window parentWindow, Variables variables, Connections connections)
        {
            try
            {
                ReplacingDataConversionEditor editor = new ReplacingDataConversionEditor(this._componentMetadata);
                if (editor.ShowDialog(parentWindow) == DialogResult.OK)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (Exception ex)
            {
                // TODO: MessageStrings
                MessageBox.Show(parentWindow, ex.Message + Environment.NewLine + ex.StackTrace,
                    "Exception in Replacing Data Conversion Editor", MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw;
            }
        }

        /// <summary>
        /// For future use in providing help for use of the component,
        /// not currently implemented
        /// </summary>
        /// <param name="parentWindow">The handle of the parent window hosting the component</param>
        public void Help(IWin32Window parentWindow)
        {

        }

        /// <summary>
        /// Handles when the component is first added to the SSIS design surface,
        /// not currently implemented
        /// </summary>
        /// <param name="parentWindow">The handle of the parent window hosting the component</param>
        public void New(IWin32Window parentWindow)
        {

        }
        #endregion
    }
}